
capture program drop prepare_quasi
program define prepare_quasi
	
	version 11

	set more off

	syntax , tfxfile(string) tch_sch_weights(string) sch_gr_yr_collapse(string) [ drop_empty_tv_cells trim_threshold(integer 100) entry_exit tv_sd ]

	use `tfxfile', clear
	
	capture confirm variable middle
	if (_rc==0) {
		* Collapse to tym, but carefully preserve middle variable, setting it =1 for teachers who teach in both elem & middle during a single year-subject
		bys teacher year middle math: keep if _n==1
		bys teacher year math: egen temp=sd(middle)
		gen midmath=(temp>0 & temp<.)
		
		replace middle=. if midmath==1 & tv==.
		
		collapse (mean) tv* (max) middle, by(teacher year math)
		replace middle=1 if middle==.
	}
	
	* Standardize tv if requested (requires middle)
	if ("`tv_sd'"!="") {
		merge m:1 middle math using ${datadir}/tfx/sd_tfx_${tfxdate}, assert(3) nogen
		foreach var of varlist tv* {
			replace `var'=`var'/sd_tfx
		}
	}

	* Create vars
	egen grp = group(teacher math)
	tsset grp year
	g tv_2yr_mean_l = (tv_2yr_l+l.tv_2yr_f)/2
	g tv_2yr_mean_f = (f.tv_2yr_l+tv_2yr_f)/2
	replace tv_2yr_mean_l = tv_2yr_l if tv_2yr_mean_l==.
	replace tv_2yr_mean_f = tv_2yr_f if tv_2yr_mean_f==.
		
	merge 1:m teacher year math using `tch_sch_weights'
	if ("`drop_empty_tv_cells'"!="") keep if _merge==3
	drop _merge

	if (`trim_threshold'!=100) {
		*Trim VA outliers (requires middle)
		foreach var of varlist tv_2yr_l tv_2yr_f {
			*Generate percentile of tv by middle-math
			g int percent_tv=.

			forvalues middle=0/1 {
				forvalues math=0/1 {
					fastxtile percent_tv_temp=`var' if middle==`middle' & math==`math' [w=n_students], nq(100)
					replace percent_tv=percent_tv_temp if middle==`middle' & math==`math'
					drop percent_tv_temp
				}
			}
			
			* Perform trimming
			replace `var' = . if percent_tv>`trim_threshold'
			drop percent_tv
		}
	}

	* Compute the weight in each cell of teachers with non-missing tv
	g temp = teach_wt
	replace temp=0 if tv==.
	bys sch grade year math: egen mean_covered_tv = sum(temp)
	drop temp
	
	
	***Define cross-cohort change in value added using 2-year exclusion
	capture confirm variable t_exp_reg
	if (_rc==0) local t_exp_reg="t_exp_reg"
	
	foreach vary of varlist tv_2yr_l tv_2yr_f `t_exp_reg' tv_2yr_mean_l tv_2yr_mean_f {
	   g temp = `vary'*teach_wt
	   by sch grade year math: egen mean_`vary' = sum(temp)
	   drop temp
	}
		
	* Code for IV with entry/exit
	if ("`entry_exit'"=="entry_exit") {
		foreach type in d s g {
			g temp = tv_2yr_l*teach_wt
			bys sch grade year math: egen mean_tv_2yr_l_`type' = sum(cond(new_to_`type'==1,temp,0))
			drop temp
			
			g temp = tv_2yr_f*teach_wt
			bys sch grade year math: egen mean_tv_2yr_f_`type' = sum(cond(switch_`type'==1,temp,0))
			drop temp
		}
		
		g temp = tv_2yr_l*teach_wt
		bys sch grade year math: egen mean_tv_2yr_l_n = sum(cond(new_to_d~=1 & new_to_s~=1 & new_to_g~=1,temp,0))
		drop temp
		
		g temp = tv_2yr_f*teach_wt
		bys sch grade year math: egen mean_tv_2yr_f_n = sum(cond(switch_d~=1 & switch_s~=1 & switch_g~=1,temp,0))
		drop temp
	}
	else {
		cap drop new_to_*
		cap drop switch_*
	}

	* Collapse to school-grade-year level
	by sch grade year math: keep if _n==1
	egen sgm_id = group(sch grade math)
	tsset sgm_id year
	g d_tv_sgm_1 = mean_tv_2yr_l-l.mean_tv_2yr_f
	g d_tv_sgm_mean = mean_tv_2yr_mean_l-l.mean_tv_2yr_mean_f

	* Entry/exit code
	if ("`entry_exit'"=="entry_exit") {
		foreach type in d s g n {
			g d_tv_sgm_1_`type' = mean_tv_2yr_l_`type'-l.mean_tv_2yr_f_`type'
		}
		
		* Test that entry/exit measures sum correctly
		egen test=rsum(d_tv_sgm_1_*)
		corr test d_tv_sgm_1
		assert round(r(rho),.0000001)==1
		drop test
	}
	
	* Other-subject d_tv_sgm_1 code
	gen d_tv_sgm_1_read=d_tv_sgm_1 if math==0
	gen d_tv_sgm_1_math=d_tv_sgm_1 if math==1
	
	isid sch grade year math
	bys sch grade year: egen d_tv_sgm_1_read2=max(d_tv_sgm_1_read)
	bys sch grade year: egen d_tv_sgm_1_math2=max(d_tv_sgm_1_math)
	
	gen d_tv_sgm_other=d_tv_sgm_1_read2 if math==1
	replace d_tv_sgm_other=d_tv_sgm_1_math2 if math==0
	
	drop d_tv_sgm_1_read d_tv_sgm_1_read2 d_tv_sgm_1_math d_tv_sgm_1_math2

	****Merge on school-grade-year collapse
	merge 1:1 sch grade year math using `sch_gr_yr_collapse'
	keep if _merge==3
	keep if grade>3
	g cohort= year-grade
	egen sch_cohort = group(sch cohort)

	sort sgm_id year
	
	g d_mean_score = d.mean_score
	
	capture confirm variable mean_other
	if (_rc==0) g d_mean_other = d.mean_other
	
	capture confirm variable pred_score_nocontrols
	if (_rc==0) g d_pred_score_nocontrols = d.pred_score_nocontrols
	
	capture confirm variable college20
	if (_rc==0) g d_college20 = d.college20
	
	capture confirm variable cwage20
	if (_rc==0) g d_cwage20 = d.cwage20
	
	capture confirm variable pctcollgrad28
	if (_rc==0) g d_pctcollgrad28 = d.pctcollgrad28
	
end
